/*  USB HID Talker source code provided by Rob Owings Quality Thermistor, Inc. for use with directemp USB-HID probes
 *  
 *  This code framework template (heavy lifting) was originally produced by wimar: 
 * http://www.codeproject.com/KB/cs/USB_HID.aspx?fid=398968&df=90&mpp=25&noise=3&sort=Position&view=Quick&select=2820233
 *  and is licensed under the code project open license:
 *  http://www.codeproject.com/info/cpol10.aspx
 * 
 * This sample program is intended as an example of how to communicate with the DirecTemp(TM) USB-HID products
 *  Please email directemp@thermistor.com with questions regarding this program
 * 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using UsbLibrary;

namespace UsbApp
{
    public partial class USB_HID : Form{
        public USB_HID(){
            InitializeComponent();
        }
        // Device is found event
        private void usb_OnDeviceArrived(object sender, EventArgs e){
            this.lb_message.Items.Add("Found Device");
        }
        // Device is removed event
        private void usb_OnDeviceRemoved(object sender, EventArgs e){
            if (InvokeRequired){
                Invoke(new EventHandler(usb_OnDeviceRemoved), new object[] { sender, e });
            }
            else{
                this.lb_message.Items.Add("Device removed");
            }
        }
        // event for specific device
        private void usb_OnSpecifiedDeviceArrived(object sender, EventArgs e){
            this.lb_message.Items.Add("Device found");
            //setting string form for sending data
            string text = "";
            for (int i = 0; i < this.usb.SpecifiedDevice.OutputReportLength; i++){
                text += "000 ";
            }
            tb_send.Text = text;
        }
        // event for device handle
        protected override void OnHandleCreated(EventArgs e){
            base.OnHandleCreated(e);
            usb.RegisterHandle(Handle);
        }
        // windows process message
        protected override void WndProc(ref Message m){
            usb.ParseMessages(ref m);
            base.WndProc(ref m);	// pass message on to base form
        }
        // connect button is pressed to look for device PID and VID - Serial number could be used as well, but isn't implemented
        private void btn_ok_Click(object sender, EventArgs e){
            try{
                this.usb.ProductId = Int32.Parse(this.tb_product.Text, System.Globalization.NumberStyles.HexNumber);
                this.usb.VendorId = Int32.Parse(this.tb_vendor.Text, System.Globalization.NumberStyles.HexNumber);
                this.usb.CheckDevicePresent();
            }
            catch (Exception ex){
                MessageBox.Show(ex.ToString());
            }
        }
        // send the data array from the text box to the device
        private void btn_send_Click(object sender, EventArgs e){
           Send(this.tb_send.Text + " ");
           tbSent.Text = tb_send.Text + "\r\n" + tbSent.Text;
        }
        // if the device is pulled, show removed event
        private void usb_OnSpecifiedDeviceRemoved(object sender, EventArgs e){
            if (InvokeRequired){
                Invoke(new EventHandler(usb_OnSpecifiedDeviceRemoved), new object[] { sender, e });
            }
            else{
                this.lb_message.Items.Add("Device was removed");
            }
        }
        // if data is received event
        private void usb_OnDataRecieved(object sender, DataRecievedEventArgs args){
            int k = 1;
            if (InvokeRequired){
                try{
                    Invoke(new DataRecievedEventHandler(usb_OnDataRecieved), new object[] { sender, args });
                }
                catch (Exception ex){
                    Console.WriteLine(ex.ToString());
                }
            }
            else{
                //string TextLine = "";
                string rec_data = "Data: ";
                // parse the incomming array byte by byte
                foreach (byte myData in args.data){
                    if (myData.ToString().Length == 1){
                        rec_data += "00";
                    }
                    if (myData.ToString().Length == 2){
                        rec_data += "0";
                    }
                    rec_data += myData.ToString() + " ";
                    if (k <= 2) {
                        switch ((char)myData){
                            case '2':
                                // message here for single data point if needed
                                break;
                            case '3':
                                // message here for information if needed
                                break;
                            case '4':
                                // message here for serial number if needed
                                break;
                        }
                    }else{    
                        tbText.Text += (char)myData;
                    }
                    k++;
                }
                this.lb_read.Items.Insert(0, rec_data);
            }
        }
        private void usb_OnDataSend(object sender, EventArgs e){
            this.lb_message.Items.Add("Data sent");
        }
        private void bSingle_Click(object sender, EventArgs e){
            Send("000 050 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000");
        }
        private void bInfo_Click(object sender, EventArgs e){
            Send("000 051 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000");
        }
        private void bSerial_Click(object sender, EventArgs e){
            Send("000 052 000 000 000 000 000 000 000 000 000 000 000 000 000 000 000");
        }
        private void Send(string text){
            try{
                text.Trim();
                tbSent.Text = text + "\r\n" + tbSent.Text;
                string[] arrText = text.Split(' ');
                byte[] data = new byte[arrText.Length];
                for (int i = 0; i < arrText.Length; i++){
                    if (arrText[i] != ""){
                        int value = Int32.Parse(arrText[i], System.Globalization.NumberStyles.Number);
                        data[i] = (byte)Convert.ToByte(value);
                    }
                }
                if (this.usb.SpecifiedDevice != null){
                    this.usb.SpecifiedDevice.SendData(data);
                }
                else{
                    MessageBox.Show("Device not present!! ");
                }
            }
            catch (Exception ex){
                MessageBox.Show(ex.ToString());
            }
        }
        private void bClear_Click(object sender, EventArgs e){
            tbText.Text = "";
            tbSent.Text = "";
        }
        private void button1_Click(object sender, EventArgs e){
            lb_message.Items.Clear();
            lb_read.Items.Clear();
        }        
    }
}